@@ -6,7 +6,7 @@
  *	cPad driver for kernel 2.4
  *
  *  Copyright (c) 2004 Jan Steinhoff <jan.steinhoff@uni-jena.de>
- *  Copyright (c) 2004 Ron Lee (ron@debian.org)
+ *  Copyright (c) 2004, 2005, 2006 Ron Lee (ron@debian.org)
  *	rewritten for kernel 2.6
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -73,7 +73,7 @@
 #include "cpadconfig.h"
 #include "cpad.h"
 
-#define DRIVER_VERSION "v0.8"
+#define DRIVER_VERSION "v0.10"
 #define DRIVER_AUTHOR	"Rob Miller (rob@inpharmatica . co . uk), "\
 			"Ron Lee (ron@debian.org), "\
 			"Jan Steinhoff <jan.steinhoff@uni-jena.de>"
@@ -229,19 +229,35 @@ static int disable_fb;
  *	module initialisation						     *
  *****************************************************************************/
 
+#define USB_VENDOR_ID_SYNAPTICS 0x06cb
+#define USB_DEVICE_ID_CPAD      0x0003
+
+static struct usb_device_id cpad_idtable [] = {
+	{ USB_DEVICE(USB_VENDOR_ID_SYNAPTICS, USB_DEVICE_ID_CPAD) },
+	{ 0, 0 }
+};
+MODULE_DEVICE_TABLE (usb, cpad_idtable);
+
+static struct usb_driver cpad_driver = {
+	.name =		"cpad",
+	.probe =	cpad_probe,
+	.disconnect =	cpad_disconnect,
+	.id_table =	cpad_idtable,
+};
+
 static int __init cpad_init(void)
 {
-	int res;
+	int result;
 
 	cpad_procfs_init();
-	res = usb_register(&cpad_driver);
-	if (res) {
-		err("usb_register failed. Error number %d", res);
-		cpad_procfs_exit();
-		return res;
-	}
-	info(DRIVER_DESC " " DRIVER_VERSION);
-	return 0;
+
+	result = usb_register(&cpad_driver);
+	if (result == 0) {
+		info(DRIVER_DESC " " DRIVER_VERSION);
+	} else
+		err("usb_register failed, error number: %d", result);
+
+	return result;
 }
 
 static void __exit cpad_exit(void)
@@ -250,6 +266,7 @@ static void __exit cpad_exit(void)
 	cpad_procfs_exit();
 }
 
+
 module_init (cpad_init);
 module_exit (cpad_exit);
 
@@ -263,23 +280,6 @@ MODULE_LICENSE ("GPL");
  *		drivers/usb/usb-skeleton.c v1.1				     *
  *****************************************************************************/
 
-#define USB_VENDOR_ID_SYNAPTICS	0x06cb
-#define USB_DEVICE_ID_CPAD	0x0003
-
-static struct usb_device_id cpad_idtable [] = {
-	{ USB_DEVICE(USB_VENDOR_ID_SYNAPTICS, USB_DEVICE_ID_CPAD) },
-	{ 0, 0 }
-};
-MODULE_DEVICE_TABLE (usb, cpad_idtable);
-
-static struct usb_driver cpad_driver = {
-	.owner =	THIS_MODULE,
-	.name =		"cpad",
-	.probe =	cpad_probe,
-	.disconnect =	cpad_disconnect,
-	.id_table =	cpad_idtable,
-};
-
 #define MAX_DEVICES	16
 static struct cpad_context *cpad_table[MAX_DEVICES] = { 0 };
 static DECLARE_MUTEX (cpad_table_sem);
@@ -501,8 +501,7 @@ static int cpad_setup_bulk_endpoint(struct cpad_endpoint *bulk,
 	}
 
 	bulk->buffer_size = buffer_size;
-	bulk->urb->transfer_flags = (URB_NO_TRANSFER_DMA_MAP |
-					URB_ASYNC_UNLINK);
+	bulk->urb->transfer_flags = URB_NO_TRANSFER_DMA_MAP;
 	bulk->buffer = usb_buffer_alloc (cpad->udev, buffer_size, GFP_KERNEL,
 					&bulk->urb->transfer_dma);
 	if (!bulk->buffer) {
@@ -514,7 +513,7 @@ static int cpad_setup_bulk_endpoint(struct cpad_endpoint *bulk,
 	return 0;
 }
 
-static int cpad_setup_int_endpoint(struct cpad_endpoint *tp, 
+static int cpad_setup_int_endpoint(struct cpad_endpoint *tp,
 					struct cpad_context *cpad)
 {
 	size_t buffer_size;
@@ -587,7 +586,7 @@ static int cpad_setup_endpoints(struct cpad_context *cpad)
 			}
 	}
 
-	if (!(cpad->display.in.endpoint_desc && 
+	if (!(cpad->display.in.endpoint_desc &&
 	      cpad->display.out.endpoint_desc &&
 	      cpad->touchpad.in.endpoint_desc)) {
 		err("Could not find all cPad endpoints");
@@ -636,7 +635,7 @@ static inline int cpad_wait (struct cpad_context *cpad, struct file *file)
 	int res;
 
 	if (atomic_read (&cpad->display.busy)) {
-		if (file->f_flags & O_NONBLOCK) {
+		if (file == NULL || file->f_flags & O_NONBLOCK) {
 			return -EAGAIN;
 		}
 		res = cpad_wait_interruptible (cpad);
@@ -680,6 +679,7 @@ static inline int cpad_submit_display_urb(struct cpad_context *cpad)
 	return res;
 }
 
+/* structure must be locked before calling this */
 static int cpad_nlcd_function(struct cpad_context *cpad, unsigned char func,
 				unsigned char val)
 {
@@ -744,7 +744,7 @@ static void cpad_light_off(void *arg)
 	up (&cpad->sem);
 }
 
-/* flash the backlight, time in 10ms. */
+/* flash the backlight, time in 10ms. call with structure locked */
 static int cpad_flash(struct cpad_context *cpad, int time)
 {
 	struct work_struct *flash = &cpad->display.flash;
@@ -990,7 +990,6 @@ static struct file_operations cpad_fops = {
 static struct usb_class_driver cpad_class = {
 	.name =		"usb/cpad%d",
 	.fops =		&cpad_fops,
-	.mode =		S_IFCHR | S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP,
 	.minor_base =	USB_CPAD_MINOR_BASE,
 };
 
@@ -1326,9 +1325,9 @@ static struct proc_dir_entry *cpad_procfs_root;
 /* procfs prototypes */
 static int cpad_procfs_open(struct inode *inode, struct file *file);
 static int cpad_procfs_release(struct inode *inode, struct file *file);
-static int cpad_procfs_read(struct file *file, char __user *buf,
+static ssize_t cpad_procfs_read(struct file *file, char __user *buf,
 				size_t nbytes, loff_t *ppos);
-static int cpad_procfs_write(struct file *file, const char __user *buffer,
+static ssize_t cpad_procfs_write(struct file *file, const char __user *buffer,
 				size_t count, loff_t *ppos);
 
 struct file_operations cpad_proc_fops = {
@@ -1535,7 +1534,7 @@ static char *cpad_procfs_read_fb(struct cpad_context *cpad, char *pos)
 	return pos;
 }
 
-static int cpad_procfs_read(struct file *file, char __user *buf,
+static ssize_t cpad_procfs_read(struct file *file, char __user *buf,
 				size_t nbytes, loff_t *ppos)
 {
 	struct cpad_context *cpad = (struct cpad_context *)file->private_data;
@@ -1604,7 +1603,7 @@ static inline void cpad_procfs_write_command(struct cpad_context *cpad,
 	}
 }
 
-static int cpad_procfs_write(struct file *file, const char __user *buffer,
+static ssize_t cpad_procfs_write(struct file *file, const char __user *buffer,
 				size_t count, loff_t *ppos)
 {
 	struct cpad_context *cpad = (struct cpad_context *)file->private_data;
@@ -1714,93 +1713,6 @@ static int cpad_fb_dither8[8][8] = 	{{ 1, 33,  9, 41,  3, 35, 11, 43},
 					 {16, 48,  8, 40, 14, 46,  6, 38},
 					 {64, 32, 56, 24, 62, 30, 54, 22}};
 
-/* frame buffer device prototypes */
-static int cpad_fb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
-					u_int transp, struct fb_info *info);
-static int cpad_fb_open(struct fb_info *info, int user);
-static int cpad_fb_release(struct fb_info *info, int user);
-static int cpad_fb_mmap(struct fb_info *info, struct file *file,
-			struct vm_area_struct *vma);
-static void cpad_fb_sendimage(void *arg);
-int cpad_fb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
-				unsigned long arg, struct fb_info *info);
-int cpad_fb_check_var(struct fb_var_screeninfo *var, struct fb_info *info);
-int cpad_fb_set_par(struct fb_info *info);
-
-static struct fb_ops cpad_fb_ops = {
-	.owner		= THIS_MODULE,
-	.fb_setcolreg	= cpad_fb_setcolreg,
-	.fb_check_var	= cpad_fb_check_var,
-	.fb_set_par	= cpad_fb_set_par,
-	.fb_fillrect	= cfb_fillrect,
-	.fb_copyarea	= cfb_copyarea,
-	.fb_imageblit	= cfb_imageblit,
-	.fb_cursor	= soft_cursor,
-	.fb_open	= cpad_fb_open,
-	.fb_release	= cpad_fb_release,
-	.fb_mmap	= cpad_fb_mmap,
-	.fb_ioctl	= cpad_fb_ioctl,
-};
-
-static void cpad_fb_add(struct cpad_context *cpad)
-{
-	struct fb_info *info = &cpad->fb.info;
-	u_long vsize, size;
-	void *adr;
-
-	if (disable_fb)
-		return;
-
-	if (cpad->table_index < 0) {
-		err("cpad_table is full, framebuffer disabled");
-		cpad->fb.disable = 1;
-		return;
-	}
-
-	if (max_bpp == 1) {
-		vsize = 8192;
-	} else {
-		vsize = 118784;
-	}
-
-	cpad_fb_default_fix.smem_len = vsize;
-	if (!(cpad->fb.videomemory = vmalloc(vsize))) {
-		cpad->fb.disable = 1;
-		err("not enough memory for frame buffer");
-		return;
-	}
-
-	memset(cpad->fb.videomemory, 0, vsize);
-	adr = cpad->fb.videomemory;
-	size = vsize;
-	while (size > 0) {
-		SetPageReserved(vmalloc_to_page(adr));
-		adr += PAGE_SIZE;
-		size -= PAGE_SIZE;
-	}
-
-	info->screen_base = cpad->fb.videomemory;
-	info->fbops = &cpad_fb_ops;
-	info->var = cpad_fb_default_var;
-	info->fix = cpad_fb_default_fix;
-	info->pseudo_palette = &cpad->fb.pseudo_palette;
-	info->flags = FBINFO_FLAG_DEFAULT;
-	fb_alloc_cmap(&cpad->fb.info.cmap, 16, 0);
-	info->par = cpad;
-
-	if (register_framebuffer(&cpad->fb.info) < 0) {
-		cpad_fb_free (cpad);
-		cpad->fb.disable = 1;
-		err("could not register frame buffer");
-		return;
-	}
-
-	cpad->fb.node = info->node;
-	cpad->fb.brightness = 200;
-	cpad->fb.dither = 3;
-	cpad->fb.onlyvisible = 1;
-}
-
 static void cpad_fb_remove(struct cpad_context *cpad)
 {
 	if (disable_fb || cpad->fb.disable)
@@ -1828,6 +1740,35 @@ static void cpad_fb_free (struct cpad_context *cpad)
 	cpad->fb.videomemory = 0;
 }
 
+static void cpad_fb_sendimage(void *arg)
+{
+	struct cpad_context *cpad = (struct cpad_context *) arg;
+
+	if (!cpad->fb.active) {
+		complete(&cpad->fb.finished);
+		return;
+	}
+
+	if (down_trylock (&cpad->sem)) {
+		INIT_WORK (&cpad->fb.sendimage, cpad_fb_sendimage, cpad);
+		schedule_delayed_work (&cpad->fb.sendimage, HZ/60);
+		return;
+	}
+
+	if (atomic_read (&cpad->display.busy))
+		wait_for_completion (&cpad->display.finished);
+
+	if (cpad_fb_oneframe(cpad)) {
+		cpad->fb.active = 0;
+		cpad->fb.rate = 0;
+	} else {
+		INIT_WORK (&cpad->fb.sendimage, cpad_fb_sendimage, cpad);
+		schedule_delayed_work (&cpad->fb.sendimage, HZ/cpad->fb.rate);
+	}
+
+	up (&cpad->sem);
+}
+
 static void cpad_fb_activate(struct cpad_context *cpad, int rate)
 {
 	if (disable_fb || cpad->fb.disable)
@@ -1919,8 +1860,7 @@ error:
 	return retval;
 }
 
-static int cpad_fb_mmap(struct fb_info *info, struct file *file,
-			struct vm_area_struct *vma)
+static int cpad_fb_mmap(struct fb_info *info, struct vm_area_struct *vma)
 {
 	unsigned long page, pos;
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)
@@ -1996,11 +1936,13 @@ static inline int cpad_fb_dither(int x, int y, int grey, int maxgrey,
 							int dither)
 {
 	switch (dither) {
-	case 0:	return (2*grey > maxgrey) ? 0 : 1;
-	case 1:	return (5*grey > cpad_fb_dither2[x%2][y%2]*maxgrey) ? 0 : 1;
-	case 2:	return (17*grey > cpad_fb_dither4[x%4][y%4]*maxgrey) ? 0 : 1;
-	case 3:	return (65*grey > cpad_fb_dither8[x%8][y%8]*maxgrey) ? 0 : 1;
+	 case 0: return (2*grey > maxgrey) ? 0 : 1;
+	 case 1: return (5*grey > cpad_fb_dither2[x%2][y%2]*maxgrey) ? 0 : 1;
+	 case 2: return (17*grey > cpad_fb_dither4[x%4][y%4]*maxgrey) ? 0 : 1;
+	 case 3: return (65*grey > cpad_fb_dither8[x%8][y%8]*maxgrey) ? 0 : 1;
 	}
+	err("dither values %d out of range", dither);
+	return 0;
 }
 
 static inline unsigned char *cpad_fb_convert_line(struct cpad_context *cpad,
@@ -2126,9 +2068,11 @@ static inline void cpad_fb_oneframe_1bpp_onlychanged(struct cpad_context *cpad,
 	}
 
 	for (line=0; line<maxline; line++) {
-		out_buf = cpad_fb_fillpacket_onlychanged (
-				param, (line==273) ? 2 : 30, out_buf,
-				cpad->fb.invert, &changed);
+		out_buf = cpad_fb_fillpacket_onlychanged (param,
+                                                          (line==273) ? 2 : 30,
+                                                          out_buf,
+                                                          cpad->fb.invert,
+                                                          &changed);
 		param += 30;
 		if (changed) {
 			if (gotfirstline) {
@@ -2163,7 +2107,8 @@ static inline void cpad_fb_oneframe_24bpp_onlychanged(struct cpad_context *cpad,
 	for (line=0; line<160; line++) {
 		vmem = cpad_fb_convert_line (cpad, line, vmem, param);
 		out_buf = cpad_fb_fillpacket_onlychanged (param, 30, out_buf,
-						cpad->fb.invert, &changed);
+                                                          cpad->fb.invert,
+                                                          &changed);
 		if (changed) {
 			if (gotfirstline) {
 				lastline = line;
@@ -2186,7 +2131,7 @@ static int cpad_fb_oneframe(struct cpad_context *cpad)
 {
 	int res;
 	int urb_size;
-	int firstline;
+	int firstline = 0;
 	int bpp = cpad->fb.info.var.bits_per_pixel;
 
 	if (cpad->fb.onlychanged) {
@@ -2223,35 +2168,6 @@ static int cpad_fb_oneframe(struct cpad_context *cpad)
 	}
 }
 
-static void cpad_fb_sendimage(void *arg)
-{
-	struct cpad_context *cpad = (struct cpad_context *) arg;
-
-	if (!cpad->fb.active) {
-		complete(&cpad->fb.finished);
-		return;
-	}
-
-	if (down_trylock (&cpad->sem)) {
-		INIT_WORK (&cpad->fb.sendimage, cpad_fb_sendimage, cpad);
-		schedule_delayed_work (&cpad->fb.sendimage, HZ/60);
-		return;
-	}
-
-	if (atomic_read (&cpad->display.busy))
-		wait_for_completion (&cpad->display.finished);
-
-	if (cpad_fb_oneframe(cpad)) {
-		cpad->fb.active = 0;
-		cpad->fb.rate = 0;
-	} else {
-		INIT_WORK (&cpad->fb.sendimage, cpad_fb_sendimage, cpad);
-		schedule_delayed_work (&cpad->fb.sendimage, HZ/cpad->fb.rate);
-	}
-
-	up (&cpad->sem);
-}
-
 static int cpad_fb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
 					u_int transp, struct fb_info *info)
 {
@@ -2268,15 +2184,14 @@ static int cpad_fb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
 	return 0;
 }
 
-int cpad_fb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
-				unsigned long arg, struct fb_info *info)
+int cpad_fb_ioctl(struct fb_info *info, unsigned int cmd, unsigned long arg)
 {
 	struct cpad_context *cpad = (struct cpad_context *)info->par;
 	unsigned char cval = 0;
 	int ival = 0;
 	int res = 0;
 
-	res = cpad_down (&cpad->sem, file);
+	res = cpad_down (&cpad->sem, NULL);
 	if (res)
 		return res;
 
@@ -2392,6 +2307,82 @@ int cpad_fb_set_par(struct fb_info *info)
 	return 0;
 }
 
+static struct fb_ops cpad_fb_ops = {
+	.owner		= THIS_MODULE,
+	.fb_setcolreg	= cpad_fb_setcolreg,
+	.fb_check_var	= cpad_fb_check_var,
+	.fb_set_par	= cpad_fb_set_par,
+	.fb_fillrect	= cfb_fillrect,
+	.fb_copyarea	= cfb_copyarea,
+	.fb_imageblit	= cfb_imageblit,
+#ifdef CONFIG_FB_SOFT_CURSOR
+	.fb_cursor	= soft_cursor,
+#endif
+	.fb_open	= cpad_fb_open,
+	.fb_release	= cpad_fb_release,
+	.fb_mmap	= cpad_fb_mmap,
+	.fb_ioctl	= cpad_fb_ioctl,
+};
+
+static void cpad_fb_add(struct cpad_context *cpad)
+{
+	struct fb_info *info = &cpad->fb.info;
+	u_long vsize, size;
+	void *adr;
+
+	if (disable_fb)
+		return;
+
+	if (cpad->table_index < 0) {
+		err("cpad_table is full, framebuffer disabled");
+		cpad->fb.disable = 1;
+		return;
+	}
+
+	if (max_bpp == 1) {
+		vsize = 8192;
+	} else {
+		vsize = 118784;
+	}
+
+	cpad_fb_default_fix.smem_len = vsize;
+	if (!(cpad->fb.videomemory = vmalloc(vsize))) {
+		cpad->fb.disable = 1;
+		err("not enough memory for frame buffer");
+		return;
+	}
+
+	memset(cpad->fb.videomemory, 0, vsize);
+	adr = cpad->fb.videomemory;
+	size = vsize;
+	while (size > 0) {
+		SetPageReserved(vmalloc_to_page(adr));
+		adr += PAGE_SIZE;
+		size -= PAGE_SIZE;
+	}
+
+	info->screen_base = cpad->fb.videomemory;
+	info->fbops = &cpad_fb_ops;
+	info->var = cpad_fb_default_var;
+	info->fix = cpad_fb_default_fix;
+	info->pseudo_palette = &cpad->fb.pseudo_palette;
+	info->flags = FBINFO_FLAG_DEFAULT;
+	fb_alloc_cmap(&cpad->fb.info.cmap, 16, 0);
+	info->par = cpad;
+
+	if (register_framebuffer(&cpad->fb.info) < 0) {
+		cpad_fb_free (cpad);
+		cpad->fb.disable = 1;
+		err("could not register frame buffer");
+		return;
+	}
+
+	cpad->fb.node = info->node;
+	cpad->fb.brightness = 200;
+	cpad->fb.dither = 3;
+	cpad->fb.onlyvisible = 1;
+}
+
 #else /* CONFIG_USB_CPADFB */
 
 static int disable_fb = 1;
